@@ -273,6 +273,7 @@ import DataFrame.Internal.DataFrame as Dataframe (
     columnAsDoubleVector,
     columnAsFloatVector,
     columnAsIntVector,
+    columnAsList,
     columnAsVector,
     empty,
     null,
@@ -313,6 +314,7 @@ import DataFrame.Operations.Permutation as Permutation (
 import DataFrame.Operations.Statistics as Statistics (
     correlation,
     frequencies,
+    imputeWith,
     interQuartileRange,
     mean,
     median,
