@@ -24,6 +24,7 @@ import Type.Reflection (typeRep)
 import DataFrame.Internal.Column (Column (..), isNumeric)
 import qualified DataFrame.Internal.Column as D
 import DataFrame.Internal.DataFrame (DataFrame (..), getColumn)
+import DataFrame.Internal.Expression
 import DataFrame.Operations.Core
 import qualified DataFrame.Operations.Subset as D
 import Granite
@@ -52,14 +53,14 @@ defaultPlotConfig ptype =
         }
 
 plotHistogram :: (HasCallStack) => T.Text -> DataFrame -> IO ()
-plotHistogram colName = plotHistogramWith colName (defaultPlotConfig Histogram)
+plotHistogram colName = plotHistogramWith colName 30 (defaultPlotConfig Histogram)
 
 plotHistogramWith ::
-    (HasCallStack) => T.Text -> PlotConfig -> DataFrame -> IO ()
-plotHistogramWith colName config df = do
+    (HasCallStack) => T.Text -> Int -> PlotConfig -> DataFrame -> IO ()
+plotHistogramWith colName numBins config df = do
     let values = extractNumericColumn colName df
         (minVal, maxVal) = if null values then (0, 1) else (minimum values, maximum values)
-    T.putStrLn $ histogram (bins 30 minVal maxVal) values (plotSettings config)
+    T.putStrLn $ histogram (bins numBins minVal maxVal) values (plotSettings config)
 
 plotScatter :: (HasCallStack) => T.Text -> T.Text -> DataFrame -> IO ()
 plotScatter xCol yCol = plotScatterWith xCol yCol (defaultPlotConfig Scatter)
@@ -82,7 +83,7 @@ plotScatterByWith xCol yCol grouping config df = do
     let vals = extractStringColumn grouping df
     let df' = insertColumn grouping (D.fromList vals) df
     xs <- forM (L.nub vals) $ \col -> do
-        let filtered = D.filter grouping (== col) df'
+        let filtered = D.filter (Col grouping) (== col) df'
             xVals = extractNumericColumn xCol filtered
             yVals = extractNumericColumn yCol filtered
             points = zip xVals yVals
