@@ -29,6 +29,7 @@ import DataFrame.Internal.DataFrame (DataFrame (..), getColumn)
 import DataFrame.Internal.Expression
 import DataFrame.Operations.Core
 import qualified DataFrame.Operations.Subset as D
+import Numeric (showFFloat)
 import System.Directory
 import System.Info
 import System.Process (
@@ -40,7 +41,6 @@ import System.Process (
     std_out,
     waitForProcess,
  )
-import Text.Printf
 
 newtype HtmlPlot = HtmlPlot T.Text deriving (Show)
 
@@ -73,10 +73,6 @@ defaultPlotConfig ptype =
         , plotFile = Nothing
         }
 
-chartJsScript :: T.Text
-chartJsScript =
-    "<script src=\"https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.js\"></script>\n"
-
 generateChartId :: IO T.Text
 generateChartId = do
     gen <- newStdGen
@@ -96,11 +92,9 @@ wrapInHTML chartId content width height =
         , "px;height:"
         , T.pack (show height)
         , "px\"></canvas>\n"
-        , "<script src=\"https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.6/require.min.js\"></script>\n"
+        , "<script src=\"https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js\"></script>\n"
         , "<script>\n"
-        , "setTimeout(() => {"
         , content
-        , "}, 200);"
         , "\n</script>\n"
         ]
 
@@ -113,13 +107,17 @@ plotHistogramWith colName numBins config df = do
     chartId <- generateChartId
     let values = extractNumericColumn colName df
         (minVal, maxVal) = if null values then (0, 1) else (minimum values, maximum values)
-        numBins = 30
         binWidth = (maxVal - minVal) / fromIntegral numBins
         bins = [minVal + fromIntegral i * binWidth | i <- [0 .. numBins - 1]]
         counts = calculateHistogram values bins binWidth
+        precision = max 0 $ ceiling (negate $ logBase 10 binWidth)
 
         labels =
-            T.intercalate "," ["\"" <> T.pack (printf "%.2f" b) <> "\"" | b <- bins]
+            T.intercalate
+                ","
+                [ "\"" <> T.pack (showFFloat (Just precision) b "") <> "\""
+                | b <- bins
+                ]
         dataPoints = T.intercalate "," [T.pack (show c) | c <- counts]
 
         chartTitle =
@@ -129,11 +127,9 @@ plotHistogramWith colName numBins config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -159,7 +155,7 @@ plotHistogramWith colName numBins config df = do
                 , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -192,11 +188,9 @@ plotScatterWith xCol yCol config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"scatter\",\n"
                 , "  data: {\n"
                 , "    datasets: [{\n"
@@ -223,7 +217,7 @@ plotScatterWith xCol yCol config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -287,11 +281,9 @@ plotScatterByWith xCol yCol grouping config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"scatter\",\n"
                 , "  data: {\n"
                 , "    datasets: [\n"
@@ -311,7 +303,7 @@ plotScatterByWith xCol yCol grouping config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -363,11 +355,9 @@ plotLinesWith xAxis colNames config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"line\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -387,7 +377,7 @@ plotLinesWith xAxis colNames config df = do
                 , "\" } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -419,11 +409,9 @@ plotSingleBars colName config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
+                        , "\", {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -447,7 +435,7 @@ plotSingleBars colName config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
@@ -465,11 +453,8 @@ plotSingleBars colName config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
                         , "\", {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
@@ -494,7 +479,7 @@ plotSingleBars colName config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
@@ -519,11 +504,9 @@ plotPieWith valCol labelCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
+                        , "\", {\n"
                         , "  type: \"pie\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -543,7 +526,7 @@ plotPieWith valCol labelCol config df = do
                         , chartTitle
                         , "\" }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
@@ -565,11 +548,9 @@ plotPieWith valCol labelCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
+                        , "\", {\n"
                         , "  type: \"pie\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -589,7 +570,7 @@ plotPieWith valCol labelCol config df = do
                         , chartTitle
                         , "\" }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
@@ -658,11 +639,9 @@ plotStackedBarsWith categoryCol valueColumns config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -681,7 +660,7 @@ plotStackedBarsWith categoryCol valueColumns config df = do
                 , "      yAxes: [{ stacked: true, ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -712,11 +691,9 @@ plotBoxPlotsWith colNames config df = do
 
         jsCode =
             T.concat
-                [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                , "var ctx = document.getElementById('"
+                [ "setTimeout(function() { new Chart(\""
                 , chartId
-                , "').getContext('2d');\n"
-                , "new Chart(ctx , {\n"
+                , "\", {\n"
                 , "  type: \"bar\",\n"
                 , "  data: {\n"
                 , "    labels: ["
@@ -740,7 +717,7 @@ plotBoxPlotsWith colNames config df = do
                 , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                 , "    }\n"
                 , "  }\n"
-                , "});});"
+                , "})}, 100);"
                 ]
 
     return $
@@ -773,11 +750,9 @@ plotGroupedBarsWithN n groupCol valCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
+                        , "\", {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -803,7 +778,7 @@ plotGroupedBarsWithN n groupCol valCol config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
@@ -827,11 +802,9 @@ plotGroupedBarsWithN n groupCol valCol config df = do
 
                 jsCode =
                     T.concat
-                        [ "requirejs(['https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.9.4/Chart.min.js'], function (Chart) {\n"
-                        , "var ctx = document.getElementById('"
+                        [ "setTimeout(function() { new Chart(\""
                         , chartId
-                        , "').getContext('2d');\n"
-                        , "new Chart(ctx , {\n"
+                        , "\", {\n"
                         , "  type: \"bar\",\n"
                         , "  data: {\n"
                         , "    labels: ["
@@ -855,7 +828,7 @@ plotGroupedBarsWithN n groupCol valCol config df = do
                         , "      yAxes: [{ ticks: { beginAtZero: true } }]\n"
                         , "    }\n"
                         , "  }\n"
-                        , "});});"
+                        , "})}, 100);"
                         ]
             return $
                 HtmlPlot $
