@@ -26,6 +26,7 @@ import DataFrame.Internal.Column
 import DataFrame.Internal.DataFrame (
     DataFrame (..),
     columnAsUnboxedVector,
+    columnAsVector,
     empty,
     getColumn,
  )
@@ -35,6 +36,7 @@ import DataFrame.Internal.Statistics
 import DataFrame.Internal.Types
 import DataFrame.Operations.Core
 import DataFrame.Operations.Subset (filterJust)
+import DataFrame.Operations.Transformations (impute)
 import Text.Printf (printf)
 import Type.Reflection (typeRep)
 
@@ -96,6 +98,15 @@ mean expr df = case interpret df expr of
         Left e -> throw e
         Right xs -> mean' xs
 
+meanMaybe ::
+    forall a. (Columnable a, Real a) => Expr (Maybe a) -> DataFrame -> Double
+meanMaybe (Col name) df = (mean' . optionalToDoubleVector) (columnAsVector @(Maybe a) name df)
+meanMaybe expr df = case interpret @(Maybe a) df expr of
+    Left e -> throw e
+    Right (TColumn col) -> case toVector @(Maybe a) col of
+        Left e -> throw e
+        Right xs -> (mean' . optionalToDoubleVector) xs
+
 -- | Calculates the median of a given column as a standalone value.
 median ::
     forall a. (Columnable a, Real a, VU.Unbox a) => Expr a -> DataFrame -> Double
@@ -178,6 +189,13 @@ _getColumnAsDouble name df = case getColumn name df of
     _ -> Nothing
 {-# INLINE _getColumnAsDouble #-}
 
+optionalToDoubleVector :: (Real a) => V.Vector (Maybe a) -> VU.Vector Double
+optionalToDoubleVector =
+    VU.fromList
+        . V.foldl'
+            (\acc e -> if isJust e then realToFrac (fromMaybe 0 e) : acc else acc)
+            []
+
 -- | Calculates the sum of a given column as a standalone value.
 sum ::
     forall a. (Columnable a, Num a) => Expr a -> DataFrame -> a
@@ -198,6 +216,60 @@ sum expr df = case interpret df expr of
         Left e -> throw e
         Right xs -> VG.sum xs
 
+{- | /O(n)/ Impute missing values in a column using a derived scalar.
+
+Given
+
+* an expression @f :: 'Expr' b -> 'Expr' b@ that, when interpreted over a
+  non-nullable column, produces the same value in every row (for example a
+  mean, median, or other aggregate), and
+* a nullable column @'Expr' ('Maybe' b)@
+
+this function:
+
+1. Drops all @Nothing@ values from the target column.
+2. Interprets @f@ on the remaining non-null values.
+3. Checks that the resulting column contains a single repeated value.
+4. Uses that value to impute all @Nothing@s in the original column.
+
+==== __Throws__
+
+* 'DataFrameException' - if the column does not exist, is empty,
+
+==== __Example__
+@
+>>> :set -XOverloadedStrings
+>>> import qualified DataFrame as D
+>>> let df =
+...       D.fromNamedColumns
+...         [ ("age", D.fromList [Just 10, Nothing, Just 20 :: Maybe Int]) ]
+>>>
+>>> -- Impute missing ages with the mean of the observed ages
+>>> D.imputeWith F.mean "age" df
+-- age
+-- ----
+-- 10
+-- 15
+-- 20
+@
+-}
+imputeWith ::
+    forall b.
+    (Columnable b) =>
+    (Expr b -> Expr b) ->
+    Expr (Maybe b) ->
+    DataFrame ->
+    DataFrame
+imputeWith f col@(Col columnName) df = case interpret @b (filterJust columnName df) (f (Col @b columnName)) of
+    Left e -> throw e
+    Right (TColumn value) -> case headColumn @b value of
+        Left e -> throw e
+        Right h ->
+            if all (== h) (toList @b value)
+                then impute col h df
+                else error "Impute expression returned more than one value"
+imputeWith _ _ df = df
+
 applyStatistic ::
     (VU.Vector Double -> Double) -> T.Text -> DataFrame -> Maybe Double
 applyStatistic f name df = apply =<< _getColumnAsDouble name (filterJust name df)
@@ -259,17 +331,18 @@ summarize df =
             quartile3 = flip (VG.!) 3 <$> quantiles
             max' = flip (VG.!) 4 <$> quantiles
             iqr = (-) <$> quartile3 <*> quartile1
+            doubleColumn col = _getColumnAsDouble col (filterJust col df)
          in
             [ count
-            , mean' <$> _getColumnAsDouble name df
+            , mean' <$> doubleColumn name
             , min'
             , quartile1
             , median'
             , quartile3
             , max'
-            , sqrt . variance' <$> _getColumnAsDouble name df
+            , sqrt . variance' <$> doubleColumn name
             , iqr
-            , skewness' <$> _getColumnAsDouble name df
+            , skewness' <$> doubleColumn name
             ]
 
 -- | Round a @Double@ to Specified Precision
